// $Id: SmtpClient.cpp,v 1.9 2014/02/10 10:07:45 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "SmtpClient.h"
#include "SmtpClient.moc"
#include "Debug.h"

static const int debugLevel(1);

namespace Calendar
{
    //___________________________________________________________________________________________
    SmtpClient::SmtpClient( QObject* parent ):
        QObject( parent ),
        Counter( "SmtpClient::SmtpClient" ),
        port_( 25 ),
        socket_( this ),
        stream_( &socket_ ),
        state_( Init )
    {

        Debug::Throw( debugLevel, "SmtpClient::SmtpClient.\n" );

        // socket connections
        connect( &socket_, SIGNAL(readyRead()), this, SLOT(_readyRead()) );
        connect( &socket_, SIGNAL(connected()), this, SLOT(_connected()) );
        connect( &socket_, SIGNAL(disconnected()), this, SLOT(_disconnected()) );
        connect( &socket_, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(_errorReceived(QAbstractSocket::SocketError)) );
        connect( &socket_, SIGNAL(stateChanged(QAbstractSocket::SocketState)), this, SLOT(_stateChanged(QAbstractSocket::SocketState)) );

    }

    //___________________________________________________________________________________________
    void SmtpClient::setHost( const QString& host, int port )
    {

        // do nothing if host is unchanged
        if( host_ == host && port_ == port ) return;

        // disconnect
        if( socket_.state() == QAbstractSocket::ConnectedState )
        { socket_.disconnect(); }

        // assign host
        host_ = host;
        port_ = port;

    }

    //___________________________________________________________________________________________
    void SmtpClient::send( const QString& sender, const QString& recipient, const QString& subject, const QString& body )
    {

        if( host_.isEmpty() )
        {
            Debug::Throw( debugLevel, "SmtpClient::send - host not set. Aborted.\n" );
            return;
        }

        // store sender and recipient
        sender_ = sender;
        recipient_ = recipient;

        // build message
        QTextStream what( &message_ );
        what << tr( "To: " ) << recipient_ << endl;
        what << tr( "From: " ) << sender_ << endl;
        what << tr( "Subject: " ) << subject << endl;
        what << endl;
        what << body;

        // some conversions
        message_.replace( QString::fromLatin1( "\n" ), QString::fromLatin1( "\r\n" ) );
        message_.replace( QString::fromLatin1( "\r\n.\r\n" ), QString::fromLatin1( "\r\n..\r\n" ) );

        // disconnect
        if( socket_.state() == QAbstractSocket::ConnectedState )
        { socket_.disconnect(); }

        // connect socket
        socket_.connectToHost( host_, port_ );

    }

    //___________________________________________________________________________________________
    void SmtpClient::_stateChanged( QAbstractSocket::SocketState socketState )
    { Debug::Throw( debugLevel ) << "SmtpClient::_stateChanged - state: " << socketState << endl; }

    //___________________________________________________________________________________________
    void SmtpClient::_errorReceived( QAbstractSocket::SocketError socketError )
    { Debug::Throw( debugLevel ) << "SmtpClient::_errorReceived - error: " << socketError << endl; }

    //___________________________________________________________________________________________
    void SmtpClient::_disconnected()
    { Debug::Throw( debugLevel ) << "SmtpClient::_disconnected - disconneted. Error; "  << socket_.errorString(); }

    //___________________________________________________________________________________________
    void SmtpClient::_connected()
    { Debug::Throw( debugLevel, "SmtpClient::_connected.\n" ); }

    //___________________________________________________________________________________________
    void SmtpClient::_readyRead()
    {

        Debug::Throw( debugLevel, "SmtpClient::_readyRead.\n" );

        // read response
        QString responseLine;
        do {

            responseLine = socket_.readLine();
            response_ += responseLine;

        } while ( socket_.canReadLine() && responseLine[3] != ' ' );
        responseLine.truncate( 3 );

        // banner was ok.
        if ( state_ == Init && responseLine[0] == '2' )
        {

            stream_ << "HELO there\r\n";
            stream_.flush();

            state_ = Mail;

        } else if ( state_ == Mail && responseLine[0] == '2' ) {

            // HELO response was okay
            stream_ << "MAIL FROM: " << sender_ << "\r\n";
            stream_.flush();
            state_ = Rcpt;

        } else if ( state_ == Rcpt && responseLine[0] == '2' ) {

            stream_ << "RCPT TO: " << recipient_ << "\r\n"; //r
            stream_.flush();
            state_ = Data;

        } else if ( state_ == Data && responseLine[0] == '2' ) {

            stream_ << "DATA\r\n";
            stream_.flush();
            state_ = Body;

        } else if ( state_ == Body && responseLine[0] == '3' ) {

            stream_ << message_ << "\r\n.\r\n";
            stream_.flush();
            state_ = Quit;

        } else if ( state_ == Quit && responseLine[0] == '2' ) {

            stream_ << "QUIT\r\n";
            stream_.flush();
            state_ = Close;
            _messageSent();

        } else if ( state_ == Close ) {

            deleteLater();
            return;

        } else {

            // something broke.
            Debug::Throw( debugLevel ) << "SmtpClient::_readyRead - unexpected reply from smtp server: " << response_ << endl;
            state_ = Close;

        }

        response_.clear();

    }

    //___________________________________________________________________________________________
    void SmtpClient::_messageSent( void )
    { Debug::Throw( debugLevel, "SmtpClient::_messageSent.\n" ); }

}
