// $Id: NetworkLoad.cpp,v 1.27 2014/02/25 10:27:39 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CustomProcess.h"
#include "Debug.h"
#include "IOString.h"
#include "NetworkLoad.h"

#include <QFile>
#include <QStringList>

namespace Network
{

    //__________________________________
    NetworkLoad::NetworkLoad( void ):
        Counter( "NetworkLoad" ),
        filename_( "/proc/net/dev" ),
        routeFilename_( "/proc/net/route" ),
        sampleSize_( 0 ),
        dataSize_( 0 )
    { timer_.start(); }

    //________________________________________________
    QString NetworkLoad::toString( double value )
    {

        QString unit( QObject::tr( "bps" ) );
        if( value > 1024 ) { value/=1024; unit = QObject::tr( "kbps" ); }
        if( value > 1024 ) { value/=1024; unit = QObject::tr( "Mbps" ); }
        return QString( "%1 %2" ).arg( QString::number( value, 'f', 1 ) ).arg( unit );

    }

    //________________________________________________
    NetworkLoad::DeviceSet NetworkLoad::devices( void ) const
    {
        DeviceSet out;
        for( SampleMap::const_iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); ++iter )
        { out.insert( iter.key() ); }
        return out;
    }

    //________________________________________________
    NetworkLoad::DeviceSet NetworkLoad::connectedDevices( void ) const
    {
        Debug::Throw( "NetworkLoad::connectedDevices.\n" );

        DeviceSet out;

        // parse routes
        QFile in( routeFilename_ );
        if( !in.open( QIODevice::ReadOnly ) ) return out;

        QStringList lines( QString( in.readAll() ).split( "\n" ) );
        for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
        {

            QTextStream stream( &*iter );
            Route route;
            stream >> route;
            if( !route.valid_ ) continue;
            Debug::Throw() << "NetworkLoad::connectedDevices - "
                << route.device_ << " "
                << route.network_ << " "
                << route.gateway_ << " "
                << route.metric_
                << endl;

            if( route.device_ == "lo" ) continue;

            // should put more checks
            out.insert( route.device_ );

        }

        return out;

    }

    //________________________________________________
    void NetworkLoad::update( void )
    {

        Debug::Throw( "NetworkLoad::update.\n" );

        // open file
        QFile in( filename_ );
        if( !in.open( QIODevice::ReadOnly ) )
        {
            Debug::Throw( "MemoryLoad::update - cannot open file.\n" );
            return;
        }

        // read lines
        QStringList lines( QString( in.readAll() ).split( "\n" ) );
        for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
        {
            if( iter->indexOf( ":" ) < 0 ) continue;
            iter->replace( ":", " " );
            QTextStream buffer( &*iter, QIODevice::ReadOnly );
            Data data;
            QString device_name;
            buffer >> device_name >> data;
            data.setTime( timer_.elapsed()/1000 );

            // skip "lo" device
            if( device_name == "lo" ) continue;

            // initialize DataSamples if new
            if( sampleMap_.find( device_name ) == sampleMap_.end() ) {
                sampleMap_[ device_name ].setSampleSize( sampleSize_ );
                sampleMap_[ device_name ].setDataSize( dataSize_ );
            }

            // add current data to matching sample
            sampleMap_[ device_name ].add( data );

        }

        return;

    }

    //________________________________________________
    NetworkLoad::DataList NetworkLoad::data( const QString& device_name ) const
    {
        SampleMap::const_iterator iter( sampleMap_.find( device_name ) );
        if( iter == sampleMap_.end() ) return DataList();
        else return iter.value().data();
    }

    //________________________________________________
    void NetworkLoad::DataSamples::setSampleSize( int sampleSize )
    {
        sampleSize_ = sampleSize;
        while( sampleList_.size() > sampleSize_ )
        { sampleList_.removeFirst(); }
    }

    //________________________________________________
    void NetworkLoad::DataSamples::setDataSize( int dataSize )
    {
        dataSize_ = dataSize;
        while( dataList_.size() > dataSize_ )
        { dataList_.removeFirst(); }
    }

    //________________________________________________
    void NetworkLoad::DataSamples::add( const NetworkLoad::Data& data )
    {

        // add data to samples
        sampleList_ << data;

        // check size
        if( sampleList_.size() > sampleSize_ ) {

            Data data( sampleList_.back() - sampleList_.front() );
            if( data.valid() ) dataList_ << data;
            if( dataList_.size() > dataSize_ ) dataList_.removeFirst();
            sampleList_.removeFirst();

        }

        return;

    }

}
