// $Id: SummaryWidget.cpp,v 1.68 2014/02/18 15:15:56 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/


#include "AdjustTimeDialog.h"
#include "BaseContextMenu.h"
#include "Debug.h"
#include "Effects.h"
#include "InformationDialog.h"
#include "KernelVersion.h"
#include "SummaryConfigurationDialog.h"
#include "SummaryData.h"
#include "SummaryWidget.h"
#include "SummaryWidget.moc"
#include "TextFormat.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QDateTime>
#include <QPainter>
#include <QLayout>
#include <QProcess>
#include <QTextLayout>

namespace Summary
{

    static const unsigned int spacing( 2 );
    static const unsigned int separator( 4 );

    //_______________________________________________
    SummaryWidget::SummaryWidget( QWidget* parent ):
        MainWindow( parent ),
        networkMonitor_( this ),
        cpuMonitor_( this ),
        diskMonitor_( this ),
        temperatureMonitor_( this ),
        temperatureMenuAction_( 0 ),
        displayMask_( All ),
        delay_( 0 ),
        hostName_( Util::host( true ) ),
        refresh_( 5 )
    {

        Debug::Throw( "SummaryWidget::SummaryWidget.\n" );
        setWindowTitle( Util::windowTitle( tr( "System Monitor Summary" ) ) );

        // data size
        cpuMonitor_.setDataSize(1);
        networkMonitor_.setDataSize(1);

        //! update configuration
        _installActions();
        _configurationAction().setText( tr( "System Monitor Summary Settings..." ) );
        _updateConfiguration();

        // start timer. update every second
        timer_.start( refresh_*1000, this );

    }

    //______________________________________________________________________
    void SummaryWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "SummaryWidget::_installContextMenuActions.\n" );

        // create parent menu
        MainWindow::_installContextMenuActions( menu );

        // customize
        menu->insertSeparator( &_menuSeparator() );
        QMenu* displayMenu( new QMenu( tr( "Display Options" ) ) );
        foreach( QAction* action, displayActionGroup_->actions() )
        { displayMenu->addAction( action ); }

        menu->insertMenu( &_menuSeparator(), displayMenu );

        // cpu menu
        menu->insertSeparator( &_menuSeparator() );
        menu->insertMenu( &_menuSeparator(), &cpuMonitor_.menu() );
        menu->insertMenu( &_menuSeparator(), &diskMonitor_.menu() );

        menu->insertMenu( &_menuSeparator(), &networkMonitor_.menu() );

        temperatureMenuAction_ = menu->insertMenu( &_menuSeparator(), &temperatureMonitor_.menu() );

        cpuMonitor_.menu().setEnabled( displayMask_&Cpu );
        diskMonitor_.menu().setEnabled( displayMask_&Disk );
        networkMonitor_.menu().setEnabled( displayMask_&Network );
        temperatureMonitor_.menu().setEnabled( displayMask_&Temperature );
        temperatureMenuAction_->setVisible( temperatureMonitor_.useSensors() );

        menu->insertSeparator( &_menuSeparator() );
        menu->insertAction( &_menuSeparator(), &_adjustTimeAction() );
    }

    //______________________________________________________________________
    void SummaryWidget::timerEvent( QTimerEvent* event )
    {

        if( event->timerId() == timer_.timerId() ) update();
        else MainWindow::timerEvent( event );
        return;

    }

    //________________________________________________
    void SummaryWidget::_configuration( void )
    {

        Debug::Throw( "SummaryWidget::_configuration.\n" );

        // save configuration
        SummaryConfigurationDialog dialog;
        _configureFromDialog( dialog );

    }

    //________________________________________________
    void SummaryWidget::_updateConfiguration( void )
    {
        Debug::Throw( "SummaryWidget::_updateConfiguration.\n" );

        // update mask
        displayMask_ = (DisplayFlags)XmlOptions::get().get<int>("DISPLAY_MASK");
        _updateDisplayActions();

        // samples
        const int samples( XmlOptions::get().get<int>( "SAMPLES" ) );
        memoryLoad_.setSampleSize( samples );

        if( temperatureMenuAction_ ) temperatureMenuAction_->setVisible( temperatureMonitor_.useSensors() );

        // menu enability
        cpuMonitor_.menu().setEnabled( displayMask_ & Cpu );
        networkMonitor_.menu().setEnabled( displayMask_ & Network );
        diskMonitor_.menu().setEnabled( displayMask_ & Disk );

        // refresh
        refresh_ = XmlOptions::get().get<int>( "REFRESH_RATE" );
        timer_.start( refresh_*1000, this );

        // delay
        delay_ = XmlOptions::get().get<int>( "CLOCK_DELAY" );

        // kernel version from options
        const Main::KernelVersion kernel = (Main::KernelVersion) XmlOptions::get().get<int>( "KERNEL_VERSION" );
        memoryLoad_.setKernelVersion( kernel );

        // need to update dataSize of kernel dependent diskLoad object here,
        // in case the kernel version has changed
        diskMonitor_.setDataSize(1);

        // base class update
        MainWindow::_updateConfiguration();
        update();

    }

    //________________________________________________
    void SummaryWidget::_adjustTime( void )
    {

        Debug::Throw( "SummaryWidget::_adjustTime.\n" );

        const bool alwaysUseCommand( XmlOptions::get().get<bool>( "ALWAYS_USE_CMD" ) );
        QString command( XmlOptions::get().contains( "ADJUST_TIME_CMD" ) ? QString( XmlOptions::get().raw( "ADJUST_TIME_CMD" ) ).trimmed():"" );

        if( !( alwaysUseCommand && !command.isEmpty() ) )
        {

            // create dialog
            AdjustTimeDialog dialog( this );
            dialog.setCommand( command );
            dialog.setAlwaysUse( alwaysUseCommand );
            if( dialog.exec() == QDialog::Rejected ) return;

            // retrieve command
            command = dialog.command().trimmed();

            // check if valid
            if( command.isEmpty() )
            {
                InformationDialog( this, tr( "No command set to adjust time. Canceled." ) ).exec();
                return;
            }

            XmlOptions::get().set<bool>( "ALWAYS_USE_CMD", dialog.alwaysUse() );

        }

        // try run
        int exitCode( QProcess::execute( command ) );
        if( exitCode != 0 )
        {
            const QString buffer = QString( tr( "Command '%1' excited with code %2" ) ).arg( command ).arg( exitCode );
            InformationDialog( this, buffer ).exec();

            XmlOptions::get().set<bool>( "ALWAYS_USE_CMD", false );

        } else {
            XmlOptions::get().setRaw( "ADJUST_TIME_CMD", command );
            XmlOptions::write();
        }

        update();

        return;

    }

    //________________________________________________
    void SummaryWidget::_toggleDisplay( QAction* action )
    {

        Debug::Throw( "SummaryWidget::_toggleDisplay.\n" );
        DisplayFlag bit( displayActions_[action] );
        if( action->isChecked() ) displayMask_ |= bit;
        else displayMask_ &= ~bit;

        XmlOptions::get().set<int>("DISPLAY_MASK", displayMask_ );

        update();

    }

    //______________________________________________________________________
    void SummaryWidget::_installActions( void )
    {

        Debug::Throw( "SummaryWidget::_installActions.\n" );

        displayActionGroup_ = new QActionGroup( this );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Host" ) ), Host );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Date" ) ), Date );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Time" ) ), Time );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Battery" ) ), Acpi );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Cpu" ) ), Cpu );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Memory" ) ), Memory );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Disk" ) ), Disk );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Network" ) ), Network );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Show Temperature" ) ), Temperature );

        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        { iter.key()->setCheckable( true ); }

        displayActionGroup_->setExclusive( false );
        _updateDisplayActions();
        connect( displayActionGroup_, SIGNAL(triggered(QAction*)), SLOT(_toggleDisplay(QAction*)) );

        addAction( adjustTimeAction_ = new QAction( tr( "Adjust Time..." ), this ) );
        adjustTimeAction_->setToolTip( tr( "Run external command to adjust system date and time" ) );
        connect( adjustTimeAction_, SIGNAL(triggered()), SLOT(_adjustTime()) );

    }

    //______________________________________________________________________
    void SummaryWidget::_updateDisplayActions( void )
    {
        Debug::Throw( "SummaryWidget::_updateDisplayActions.\n" );
        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        { iter.key()->setChecked( displayMask_ & iter.value() ); }
    }

    //______________________________________________________________________
    void SummaryWidget::_paint( QPaintDevice& device, const QRect& clip )
    {
        // create painter
        QPainter painter( &device );
        painter.setClipRect( clip );
        QColor color( foregroundColor() );
        painter.setPen( color );

        // text is first drawn on a buffer pixmap
        // it is then copied to main device after proper shadow is calculated
        QPixmap pixmap( size() );
        pixmap.fill( Qt::transparent );
        QPainter textPainter( &pixmap );
        textPainter.setPen( painter.pen() );

        // prepare rect
        QRect rect( _margins().adjustedRect( SummaryWidget::rect() ) );
        const Format::TextFormatFlags format( Format::Bold );

        // hostname
        bool firstItem = true;
        QRect horizontalLineRect;
        if( (displayMask_ & Host) && hostName_.size() )
        {

            if( !firstItem ) SummaryData().setIsSeparator().paint( textPainter, rect );
            firstItem = false;

            SummaryData( "", hostName_, format ).paint( textPainter, rect );
            horizontalLineRect = rect;
            SummaryData().setIsSeparator().paint( textPainter, rect );

        }

        SummaryDataList dataList;

        // date and time settings
        QDateTime now = QDateTime::currentDateTime().addSecs( 3600* delay_ );

        // date
        if( displayMask_ & Date )
        {
            if( !firstItem ) dataList << SummaryData().setIsSeparator();
            firstItem = false;

            const QString buffer( now.date().toString( "dd/MM/yyyy" ) );
            dataList << SummaryData( tr( "Date" ), buffer, format );

        }

        // time
        if( displayMask_ & Time )
        {

            if( !( firstItem || (displayMask_ & Date) ) ) dataList << SummaryData().setIsSeparator();
            firstItem = false;

            const QString buffer = now.time().toString( "hh:mm" );
            dataList << SummaryData(  tr( "Time" ), buffer, format );

        }

        // battery data
        if( displayMask_ & Acpi )
        {

            if( !firstItem ) dataList << SummaryData().setIsSeparator();
            firstItem = false;

            acpi_.update();
            const QString buffer = QString( "%1%" ).arg( acpi_.progress() );
            dataList << SummaryData(  tr( "Battery" ), buffer, format );

        }


        // cpu data
        if( displayMask_ & Cpu )
        {

            cpuMonitor_.update();
            const Cpu::CpuLoad::DataList& data( cpuMonitor_.data() );
            if( !data.empty() )
            {

                if( !firstItem ) dataList << SummaryData().setIsSeparator();
                firstItem = false;

                const QString buffer = QString( "%1%" ).arg( data.back().ratio() );
                dataList << SummaryData(  tr( "Cpu" ), buffer, format );

            }

        }

        // memory data
        if( displayMask_ & Memory )
        {

            memoryLoad_.update();
            const Memory::MemoryLoad::DataList& data( memoryLoad_.data() );
            if( !data.empty() )
            {

                if( !firstItem ) dataList << SummaryData().setIsSeparator();
                firstItem = false;

                QString buffer = QString( "%1%" ).arg( data.back().memoryRatio() );
                dataList << SummaryData(  tr( "Memory" ), buffer, format );

                buffer = QString( "%1%" ).arg( data.back().swapRatio() );
                dataList << SummaryData(  tr( "Swap" ), buffer, format );

            }

        }

        // disk data
        if( displayMask_ & Disk )
        {

            diskMonitor_.update();
            const Disk::DataSamples::DataList& data( diskMonitor_.data() );
            if( !data.empty() )
            {

                if( !firstItem ) dataList << SummaryData().setIsSeparator();
                firstItem = false;

                const Disk::Data& last( data.back() );

                QString buffer = Disk::DiskLoad::toString( double(last.read())/last.time() );
                dataList << SummaryData( tr( "Disk Read" ), buffer, format );

                buffer = Disk::DiskLoad::toString( double(last.write())/last.time() );
                dataList << SummaryData( tr( "Disk Write" ), buffer, format );

            }

        }


        // network data
        if( displayMask_ & Network )
        {

            networkMonitor_.update();
            Network::NetworkLoad::DataList data( networkMonitor_.data() );
            if( !data.empty() )
            {

                if( !firstItem ) dataList << SummaryData().setIsSeparator();
                firstItem = false;

                const Network::NetworkLoad::Data& last( data.back() );

                QString buffer = Network::NetworkLoad::toString( double(last.bytesRecieved())/last.time() );
                dataList << SummaryData(  tr( "Network In" ), buffer, format );

                buffer = Network::NetworkLoad::toString( double(last.bytesTransmitted())/last.time() );
                dataList << SummaryData(  tr( "Network Out" ), buffer, format );

            }

        }

        // temperature data
        if( displayMask_ & Temperature )
        {

            temperatureMonitor_.update();

            const Temperature::Temperature::DataList& data( temperatureMonitor_.data() );
            if( !data.empty() )
            {

                if( !firstItem ) dataList << SummaryData().setIsSeparator();
                firstItem = false;

                const QString buffer = QString( tr( "%1 C" ) ).arg( data.back().temperature() );
                dataList << SummaryData(  tr( "Temperature:" ), buffer, format );

            }

        }

        // paint
        dataList.paint( textPainter, rect );

        int newHeight = rect.top() + _margins().top();
        int newWidth = qMax<int>( width(), rect.right() + _margins().right() + 1 );
        if( newHeight != height() || newWidth != width() )
        { resize( QSize( newWidth, newHeight ) ); }

        textPainter.end();

        if( shadowOffset() > 0 && shadowColor().isValid() )
        {
            QImage image( pixmap.toImage() );
            Transparency::Effects::shadowBlur(image, shadowOffset(), shadowColor() );
            painter.drawImage( SummaryWidget::rect().topLeft(), image);
        }

        painter.drawPixmap( SummaryWidget::rect().topLeft(), pixmap );

        if( !horizontalLineRect.isNull() )
        {
            // horizontal line
            _paintHorizontalLine( painter, horizontalLineRect.translated( SummaryWidget::rect().topLeft() ) );
        }


        painter.end();

    }

    //________________________________________________
    void SummaryWidget::_paintHorizontalLine( QPainter& painter, QRect rect ) const
    {
        painter.save();
        painter.setRenderHint(QPainter::Antialiasing, false );

        {
            QLinearGradient gradient( QPointF( _margins().left(), 0 ), QPointF( width() - _margins().right(), 0 ) );
            QColor c( "#555555" );
            gradient.setColorAt(0.3, c );
            gradient.setColorAt(0.7, c );
            c.setAlphaF( 0 ); gradient.setColorAt(0, c );
            c.setAlphaF( 0 ); gradient.setColorAt(1, c );

            painter.setBrush( Qt::NoBrush );
            painter.setPen( QPen( gradient, 1 ) );
            painter.drawLine( _margins().left(), 2+rect.top(), width()-_margins().right(), 2+rect.top() );

        }

        {
            QLinearGradient gradient( QPointF( _margins().left(), 0 ), QPointF( width() - _margins().right(), 0 ) );
            QColor c( "#cccccc" );
            gradient.setColorAt(0.3, c );
            gradient.setColorAt(0.7, c );
            c.setAlphaF( 0 ); gradient.setColorAt(0, c );
            c.setAlphaF( 0 ); gradient.setColorAt(1, c );
            painter.setBrush( Qt::NoBrush );
            painter.setPen( QPen( gradient, 1 ) );
            painter.drawLine( _margins().left(), 3+rect.top(), width()-_margins().right(), 3+rect.top() );

        }

        painter.restore();
    }

}
